﻿using System;
using HarmonyLib;
using JetBrains.Annotations;
using UnityEngine;
using UnityEngine.AzureSky;

public class LongerDayNightCycles : Mod
{
    private const string ModNamePrefix = "<color=#42a7f5>LongerDay</color><color=#FF0000>NightCycles</color>";
    private const string DayLengthArgumentsAreInvalid = "daylength: Needs one parameter.\n" +
                                                        "e.g. use \"daylength 60\" sets every day to be 60 minutes long. Default length is 20 minutes.";
    private const string DayLengthArgumentAreOutOfRange = "moreanimals: The lenght needs to be 1 and 90.";

    private float m_dayLength;
    private float m_elapsed;

    public static LongerDayNightCycles instance;

    public void Start()
    {
        instance = this;

        Debug.Log(string.Format("{0} has been loaded!", ModNamePrefix));
    }

    private void SetDaylength(string[] args)
    {
        if (args.Length != 1)
        {
            Debug.Log(DayLengthArgumentsAreInvalid);
            return;
        }

        float length;
        if ((!float.TryParse(args[0], out length)))
        {
            Debug.Log(DayLengthArgumentAreOutOfRange);
            return;
        }

        if (length < 1 ||
            length > 90)
        {
            Debug.Log(DayLengthArgumentAreOutOfRange);
            return;
        }

        m_dayLength = length;
        Debug.Log(string.Format("{0}: Sucessfully set daylength to {1} minutes.", ModNamePrefix, m_dayLength));
    }

    public void Update()
    {
        if (!LoadSceneManager.IsGameSceneLoaded)
        {
            return;
        }

        m_elapsed += Time.deltaTime;
        if (m_elapsed < 2)
        {
            return;
        }

        m_elapsed = 0;

        var skyController = FindObjectOfType<AzureSkyController>();
        if (skyController == null ||
            skyController.timeOfDay == null ||
            !(Math.Abs(skyController.timeOfDay.dayCycle - m_dayLength) > 0.01))
        {
            return;
        }

        skyController.timeOfDay.dayCycle = m_dayLength;
        Traverse.Create(skyController).Field<float>("m_timeProgression").Value =
            skyController.timeOfDay.GetDayLength();
    }

    public void OnModUnload()
    {
        Debug.Log(string.Format("{0} has been unloaded!", ModNamePrefix));
    }

    [ConsoleCommand(name: "daylength", docs: "Gives you the ability to change the duration of a day.")]
    public static void backupCommand(string[] args)
    {
        instance.SetDaylength(args);
    }
}